@@ -6,13 +6,12 @@
 import json
 import os
 from pathlib import Path
-from typing import Any, Callable, Dict, List, Optional, Tuple
+from typing import Any, Dict, List, Tuple
 
 import numpy as np
 
-from doctr.utils.geometry import fit_rbbox
-
 from .datasets import VisionDataset
+from .utils import convert_target_to_relative
 
 __all__ = ['CORD']
 
@@ -28,8 +27,7 @@ class CORD(VisionDataset):
 
     Args:
         train: whether the subset should be the training one
-        sample_transforms: composable transformations that will be applied to each image
-        rotated_bbox: whether polygons should be considered as rotated bounding box (instead of straight ones)
+        use_polygons: whether polygons should be considered as rotated bounding box (instead of straight ones)
         **kwargs: keyword arguments from `VisionDataset`.
     """
     TRAIN = ('https://github.com/mindee/doctr/releases/download/v0.1.1/cord_train.zip',
@@ -41,23 +39,22 @@ class CORD(VisionDataset):
     def __init__(
         self,
         train: bool = True,
-        sample_transforms: Optional[Callable[[Any], Any]] = None,
-        rotated_bbox: bool = False,
+        use_polygons: bool = False,
         **kwargs: Any,
     ) -> None:
 
         url, sha256 = self.TRAIN if train else self.TEST
-        super().__init__(url, None, sha256, True, **kwargs)
+        super().__init__(url, None, sha256, True, pre_transforms=convert_target_to_relative, **kwargs)
 
         # # List images
         tmp_root = os.path.join(self.root, 'image')
         self.data: List[Tuple[str, Dict[str, Any]]] = []
         self.train = train
-        self.sample_transforms = sample_transforms
         for img_path in os.listdir(tmp_root):
             # File existence check
             if not os.path.exists(os.path.join(tmp_root, img_path)):
                 raise FileNotFoundError(f"unable to locate {os.path.join(tmp_root, img_path)}")
+
             stem = Path(img_path).stem
             _targets = []
             with open(os.path.join(self.root, 'json', f"{stem}.json"), 'rb') as f:
@@ -67,13 +64,13 @@ class CORD(VisionDataset):
                         if len(word["text"]) > 0:
                             x = word["quad"]["x1"], word["quad"]["x2"], word["quad"]["x3"], word["quad"]["x4"]
                             y = word["quad"]["y1"], word["quad"]["y2"], word["quad"]["y3"], word["quad"]["y4"]
-                            if rotated_bbox:
-                                box = list(fit_rbbox(np.array([
+                            if use_polygons:
+                                box = np.array([
                                     [x[0], y[0]],
                                     [x[1], y[1]],
                                     [x[2], y[2]],
                                     [x[3], y[3]],
-                                ], dtype=np.float32)))
+                                ], dtype=np.float32)
                             else:
                                 # Reduce 8 coords to 4
                                 box = [min(x), min(y), max(x), max(y)]
@@ -83,7 +80,7 @@ class CORD(VisionDataset):
 
             self.data.append((
                 img_path,
-                dict(boxes=np.asarray(box_targets, dtype=int).clip(min=0), labels=text_targets)
+                dict(boxes=np.asarray(box_targets, dtype=int).clip(min=0), labels=list(text_targets))
             ))
         self.root = tmp_root
 
