@@ -3,16 +3,13 @@
 # This program is licensed under the Apache License version 2.
 # See LICENSE or go to <https://www.apache.org/licenses/LICENSE-2.0.txt> for full license details.
 
-from typing import List, Tuple
+from typing import List
 
 import cv2
 import numpy as np
 
-from doctr.utils.geometry import rotate_image
 from doctr.utils.repr import NestedObject
 
-from .._utils import get_bitmap_angle
-
 __all__ = ['DetectionPostProcessor']
 
 
@@ -20,30 +17,31 @@ class DetectionPostProcessor(NestedObject):
     """Abstract class to postprocess the raw output of the model
 
     Args:
-        min_size_box (int): minimal length (pix) to keep a box
-        max_candidates (int): maximum boxes to consider in a single page
         box_thresh (float): minimal objectness score to consider a box
+        bin_thresh (float): threshold to apply to segmentation raw heatmap
+        assume straight_pages (bool): if True, fit straight boxes only
     """
 
     def __init__(
         self,
         box_thresh: float = 0.5,
         bin_thresh: float = 0.5,
-        rotated_bbox: bool = False
+        assume_straight_pages: bool = True
     ) -> None:
 
         self.box_thresh = box_thresh
         self.bin_thresh = bin_thresh
-        self.rotated_bbox = rotated_bbox
+        self.assume_straight_pages = assume_straight_pages
+        self._opening_kernel = np.ones((3, 3), dtype=np.uint8)
 
     def extra_repr(self) -> str:
-        return f"box_thresh={self.box_thresh}"
+        return f"bin_thresh={self.bin_thresh}, box_thresh={self.box_thresh}"
 
     @staticmethod
     def box_score(
         pred: np.ndarray,
         points: np.ndarray,
-        rotated_bbox: bool = False
+        assume_straight_pages: bool = True
     ) -> float:
         """Compute the confidence score for a polygon : mean of the p values on the polygon
 
@@ -55,7 +53,7 @@ class DetectionPostProcessor(NestedObject):
         """
         h, w = pred.shape[:2]
 
-        if not rotated_bbox:
+        if assume_straight_pages:
             xmin = np.clip(np.floor(points[:, 0].min()).astype(np.int32), 0, w - 1)
             xmax = np.clip(np.ceil(points[:, 0].max()).astype(np.int32), 0, w - 1)
             ymin = np.clip(np.floor(points[:, 1].min()).astype(np.int32), 0, h - 1)
@@ -77,33 +75,31 @@ class DetectionPostProcessor(NestedObject):
 
     def __call__(
         self,
-        proba_map: np.ndarray,
-    ) -> Tuple[List[np.ndarray], List[float]]:
+        proba_map,
+    ) -> List[List[np.ndarray]]:
         """Performs postprocessing for a list of model outputs
 
         Args:
-            proba_map: probability map of shape (N, H, W)
+            proba_map: probability map of shape (N, H, W, C)
 
-        returns:
-            list of N tensors (for each input sample), with each tensor of shape (*, 5) or (*, 6),
-            and a list of N angles (page orientations).
+        Returns:
+            list of N class predictions (for each input sample), where each class predictions is a list of C tensors
+        of shape (*, 5) or (*, 6)
         """
 
-        bitmap = (proba_map > self.bin_thresh).astype(proba_map.dtype)
+        if proba_map.ndim != 4:
+            raise AssertionError(f"arg `proba_map` is expected to be 4-dimensional, got {proba_map.ndim}.")
 
-        boxes_batch, angles_batch = [], []
-        # Kernel for opening, empirical law for ksize
-        k_size = 1 + int(proba_map[0].shape[0] / 512)
-        kernel = np.ones((k_size, k_size), np.uint8)
+        # Erosion + dilation on the binary map
+        bin_map = [
+            [
+                cv2.morphologyEx(bmap[..., idx], cv2.MORPH_OPEN, self._opening_kernel)
+                for idx in range(proba_map.shape[-1])
+            ]
+            for bmap in (proba_map >= self.bin_thresh).astype(np.uint8)
+        ]
 
-        for p_, bitmap_ in zip(proba_map, bitmap):
-            # Perform opening (erosion + dilatation)
-            bitmap_ = cv2.morphologyEx(bitmap_.astype(np.float32), cv2.MORPH_OPEN, kernel)
-            # Rotate bitmap and proba_map
-            angle = get_bitmap_angle(bitmap_)
-            angles_batch.append(angle)
-            bitmap_, p_ = rotate_image(bitmap_, -angle, False), rotate_image(p_, -angle, False)
-            boxes = self.bitmap_to_boxes(pred=p_, bitmap=bitmap_)
-            boxes_batch.append(boxes)
-
-        return boxes_batch, angles_batch
+        return [
+            [self.bitmap_to_boxes(pmaps[..., idx], bmaps[idx]) for idx in range(proba_map.shape[-1])]
+            for pmaps, bmaps in zip(proba_map, bin_map)
+        ]
