@@ -3,7 +3,7 @@
 # This program is licensed under the Apache License version 2.
 # See LICENSE or go to <https://www.apache.org/licenses/LICENSE-2.0.txt> for full license details.
 
-from typing import Any, Dict, List, Optional
+from typing import Any, Callable, Dict, List, Optional
 
 import numpy as np
 import torch
@@ -13,41 +13,38 @@ from torchvision.models import resnet34, resnet50
 from torchvision.models._utils import IntermediateLayerGetter
 from torchvision.ops.deform_conv import DeformConv2d
 
-from ...backbones import mobilenet_v3_large
+from ...classification import mobilenet_v3_large
 from ...utils import load_pretrained_params
 from .base import DBPostProcessor, _DBNet
 
-__all__ = ['DBNet', 'db_resnet50', 'db_resnet34', 'db_mobilenet_v3_large']
+__all__ = ['DBNet', 'db_resnet50', 'db_resnet34', 'db_mobilenet_v3_large', 'db_resnet50_rotation']
 
 
 default_cfgs: Dict[str, Dict[str, Any]] = {
     'db_resnet50': {
-        'backbone': resnet50,
-        'backbone_submodule': None,
-        'fpn_layers': ['layer1', 'layer2', 'layer3', 'layer4'],
         'input_shape': (3, 1024, 1024),
         'mean': (0.798, 0.785, 0.772),
         'std': (0.264, 0.2749, 0.287),
         'url': 'https://github.com/mindee/doctr/releases/download/v0.3.1/db_resnet50-ac60cadc.pt',
     },
     'db_resnet34': {
-        'backbone': resnet34,
-        'backbone_submodule': None,
-        'fpn_layers': ['layer1', 'layer2', 'layer3', 'layer4'],
         'input_shape': (3, 1024, 1024),
         'mean': (.5, .5, .5),
         'std': (1., 1., 1.),
         'url': None,
     },
     'db_mobilenet_v3_large': {
-        'backbone': mobilenet_v3_large,
-        'backbone_submodule': 'features',
-        'fpn_layers': ['3', '6', '12', '16'],
         'input_shape': (3, 1024, 1024),
         'mean': (0.798, 0.785, 0.772),
         'std': (0.264, 0.2749, 0.287),
         'url': 'https://github.com/mindee/doctr/releases/download/v0.3.1/db_mobilenet_v3_large-fd62154b.pt',
     },
+    'db_resnet50_rotation': {
+        'input_shape': (3, 1024, 1024),
+        'mean': (0.798, 0.785, 0.772),
+        'std': (0.264, 0.2749, 0.287),
+        'url': 'https://github.com/mindee/doctr/releases/download/v0.4.1/db_resnet50-1138863a.pt',
+    },
 }
 
 
@@ -105,7 +102,7 @@ class DBNet(_DBNet, nn.Module):
         head_chans: int = 256,
         deform_conv: bool = False,
         num_classes: int = 1,
-        rotated_bbox: bool = False,
+        assume_straight_pages: bool = True,
         cfg: Optional[Dict[str, Any]] = None,
     ) -> None:
 
@@ -114,7 +111,7 @@ class DBNet(_DBNet, nn.Module):
 
         conv_layer = DeformConv2d if deform_conv else nn.Conv2d
 
-        self.rotated_bbox = rotated_bbox
+        self.assume_straight_pages = assume_straight_pages
 
         self.feat_extractor = feat_extractor
         # Identify the number of channels for the head initialization
@@ -137,7 +134,7 @@ class DBNet(_DBNet, nn.Module):
             nn.ConvTranspose2d(head_chans // 4, head_chans // 4, 2, stride=2, bias=False),
             nn.BatchNorm2d(head_chans // 4),
             nn.ReLU(inplace=True),
-            nn.ConvTranspose2d(head_chans // 4, 1, 2, stride=2),
+            nn.ConvTranspose2d(head_chans // 4, num_classes, 2, stride=2),
         )
         self.thresh_head = nn.Sequential(
             conv_layer(head_chans, head_chans // 4, 3, padding=1, bias=False),
@@ -149,9 +146,12 @@ class DBNet(_DBNet, nn.Module):
             nn.ConvTranspose2d(head_chans // 4, num_classes, 2, stride=2),
         )
 
-        self.postprocessor = DBPostProcessor(rotated_bbox=rotated_bbox)
+        self.postprocessor = DBPostProcessor(assume_straight_pages=assume_straight_pages)
 
-        for m in self.modules():
+        for n, m in self.named_modules():
+            # Don't override the initialization of the backbone
+            if n.startswith('feat_extractor.'):
+                continue
             if isinstance(m, (nn.Conv2d, DeformConv2d)):
                 nn.init.kaiming_normal_(m.weight.data, mode='fan_out', nonlinearity='relu')
                 if m.bias is not None:
@@ -165,7 +165,7 @@ class DBNet(_DBNet, nn.Module):
         x: torch.Tensor,
         target: Optional[List[np.ndarray]] = None,
         return_model_output: bool = False,
-        return_boxes: bool = False,
+        return_preds: bool = False,
     ) -> Dict[str, torch.Tensor]:
         # Extract feature maps at different stages
         feats = self.feat_extractor(x)
@@ -175,15 +175,17 @@ class DBNet(_DBNet, nn.Module):
         logits = self.prob_head(feat_concat)
 
         out: Dict[str, Any] = {}
-        if return_model_output or target is None or return_boxes:
+        if return_model_output or target is None or return_preds:
             prob_map = torch.sigmoid(logits)
 
         if return_model_output:
             out["out_map"] = prob_map
 
-        if target is None or return_boxes:
-            # Post-process boxes
-            out["preds"] = self.postprocessor(prob_map.squeeze(1).detach().cpu().numpy())
+        if target is None or return_preds:
+            # Post-process boxes (keep only text predictions)
+            out["preds"] = [
+                preds[0] for preds in self.postprocessor(prob_map.detach().cpu().permute((0, 2, 3, 1)).numpy())
+            ]
 
         if target is not None:
             thresh_map = self.thresh_head(feat_concat)
@@ -213,7 +215,7 @@ class DBNet(_DBNet, nn.Module):
         prob_map = torch.sigmoid(out_map.squeeze(1))
         thresh_map = torch.sigmoid(thresh_map.squeeze(1))
 
-        targets = self.compute_target(target, prob_map.shape)  # type: ignore[arg-type]
+        targets = self.build_target(target, prob_map.shape)  # type: ignore[arg-type]
 
         seg_target, seg_mask = torch.from_numpy(targets[0]), torch.from_numpy(targets[1])
         seg_target, seg_mask = seg_target.to(out_map.device), seg_mask.to(out_map.device)
@@ -222,48 +224,58 @@ class DBNet(_DBNet, nn.Module):
 
         # Compute balanced BCE loss for proba_map
         bce_scale = 5.
-        bce_loss = F.binary_cross_entropy_with_logits(out_map.squeeze(1), seg_target, reduction='none')[seg_mask]
+        balanced_bce_loss = torch.zeros(1, device=out_map.device)
+        dice_loss = torch.zeros(1, device=out_map.device)
+        l1_loss = torch.zeros(1, device=out_map.device)
+        if torch.any(seg_mask):
+            bce_loss = F.binary_cross_entropy_with_logits(out_map.squeeze(1), seg_target, reduction='none')[seg_mask]
 
-        neg_target = 1 - seg_target[seg_mask]
-        positive_count = seg_target[seg_mask].sum()
-        negative_count = torch.minimum(neg_target.sum(), 3. * positive_count)
-        negative_loss = bce_loss * neg_target
-        negative_loss = negative_loss.sort().values[-int(negative_count.item()):]
-        sum_losses = torch.sum(bce_loss * seg_target[seg_mask]) + torch.sum(negative_loss)
-        balanced_bce_loss = sum_losses / (positive_count + negative_count + 1e-6)
+            neg_target = 1 - seg_target[seg_mask]
+            positive_count = seg_target[seg_mask].sum()
+            negative_count = torch.minimum(neg_target.sum(), 3. * positive_count)
+            negative_loss = bce_loss * neg_target
+            negative_loss = negative_loss.sort().values[-int(negative_count.item()):]
+            sum_losses = torch.sum(bce_loss * seg_target[seg_mask]) + torch.sum(negative_loss)
+            balanced_bce_loss = sum_losses / (positive_count + negative_count + 1e-6)
 
-        # Compute dice loss for approxbin_map
-        bin_map = 1 / (1 + torch.exp(-50. * (prob_map[seg_mask] - thresh_map[seg_mask])))
+            # Compute dice loss for approxbin_map
+            bin_map = 1 / (1 + torch.exp(-50. * (prob_map[seg_mask] - thresh_map[seg_mask])))
 
-        bce_min = bce_loss.min()
-        weights = (bce_loss - bce_min) / (bce_loss.max() - bce_min) + 1.
-        inter = torch.sum(bin_map * seg_target[seg_mask] * weights)
-        union = torch.sum(bin_map) + torch.sum(seg_target[seg_mask]) + 1e-8
-        dice_loss = 1 - 2.0 * inter / union
+            bce_min = bce_loss.min()
+            weights = (bce_loss - bce_min) / (bce_loss.max() - bce_min) + 1.
+            inter = torch.sum(bin_map * seg_target[seg_mask] * weights)
+            union = torch.sum(bin_map) + torch.sum(seg_target[seg_mask]) + 1e-8
+            dice_loss = 1 - 2.0 * inter / union
 
         # Compute l1 loss for thresh_map
         l1_scale = 10.
         if torch.any(thresh_mask):
             l1_loss = torch.mean(torch.abs(thresh_map[thresh_mask] - thresh_target[thresh_mask]))
-        else:
-            l1_loss = torch.zeros(1)
 
         return l1_scale * l1_loss + bce_scale * balanced_bce_loss + dice_loss
 
 
-def _dbnet(arch: str, pretrained: bool, pretrained_backbone: bool = True, **kwargs: Any) -> DBNet:
+def _dbnet(
+    arch: str,
+    pretrained: bool,
+    backbone_fn: Callable[[bool], nn.Module],
+    fpn_layers: List[str],
+    backbone_submodule: Optional[str] = None,
+    pretrained_backbone: bool = True,
+    **kwargs: Any,
+) -> DBNet:
 
     # Starting with Imagenet pretrained params introduces some NaNs in layer3 & layer4 of resnet50
     pretrained_backbone = pretrained_backbone and not arch.split('_')[1].startswith('resnet')
     pretrained_backbone = pretrained_backbone and not pretrained
 
     # Feature extractor
-    backbone = default_cfgs[arch]['backbone'](pretrained=pretrained_backbone)
-    if isinstance(default_cfgs[arch]['backbone_submodule'], str):
-        backbone = getattr(backbone, default_cfgs[arch]['backbone_submodule'])
+    backbone = backbone_fn(pretrained_backbone)
+    if isinstance(backbone_submodule, str):
+        backbone = getattr(backbone, backbone_submodule)
     feat_extractor = IntermediateLayerGetter(
         backbone,
-        {layer_name: str(idx) for idx, layer_name in enumerate(default_cfgs[arch]['fpn_layers'])},
+        {layer_name: str(idx) for idx, layer_name in enumerate(fpn_layers)},
     )
 
     # Build the model
@@ -293,7 +305,14 @@ def db_resnet34(pretrained: bool = False, **kwargs: Any) -> DBNet:
         text detection architecture
     """
 
-    return _dbnet('db_resnet34', pretrained, **kwargs)
+    return _dbnet(
+        'db_resnet34',
+        pretrained,
+        resnet34,
+        ['layer1', 'layer2', 'layer3', 'layer4'],
+        None,
+        **kwargs,
+    )
 
 
 def db_resnet50(pretrained: bool = False, **kwargs: Any) -> DBNet:
@@ -314,7 +333,14 @@ def db_resnet50(pretrained: bool = False, **kwargs: Any) -> DBNet:
         text detection architecture
     """
 
-    return _dbnet('db_resnet50', pretrained, **kwargs)
+    return _dbnet(
+        'db_resnet50',
+        pretrained,
+        resnet50,
+        ['layer1', 'layer2', 'layer3', 'layer4'],
+        None,
+        **kwargs,
+    )
 
 
 def db_mobilenet_v3_large(pretrained: bool = False, **kwargs: Any) -> DBNet:
@@ -335,4 +361,40 @@ def db_mobilenet_v3_large(pretrained: bool = False, **kwargs: Any) -> DBNet:
         text detection architecture
     """
 
-    return _dbnet('db_mobilenet_v3_large', pretrained, **kwargs)
+    return _dbnet(
+        'db_mobilenet_v3_large',
+        pretrained,
+        mobilenet_v3_large,
+        ['3', '6', '12', '16'],
+        'features',
+        **kwargs,
+    )
+
+
+def db_resnet50_rotation(pretrained: bool = False, **kwargs: Any) -> DBNet:
+    """DBNet as described in `"Real-time Scene Text Detection with Differentiable Binarization"
+    <https://arxiv.org/pdf/1911.08947.pdf>`_, using a ResNet-50 backbone.
+    This model is trained with rotated documents
+
+    Example::
+        >>> import torch
+        >>> from doctr.models import db_resnet50_rotation
+        >>> model = db_resnet50_rotation(pretrained=True)
+        >>> input_tensor = torch.rand((1, 3, 1024, 1024), dtype=torch.float32)
+        >>> out = model(input_tensor)
+
+    Args:
+        pretrained (bool): If True, returns a model pre-trained on our text detection dataset
+
+    Returns:
+        text detection architecture
+    """
+
+    return _dbnet(
+        'db_resnet50_rotation',
+        pretrained,
+        resnet50,
+        ['layer1', 'layer2', 'layer3', 'layer4'],
+        None,
+        **kwargs,
+    )
