@@ -45,7 +45,7 @@ class DetectionPredictor(nn.Module):
 
         processed_batches = self.pre_processor(pages)
         predicted_batches = [
-            self.model(batch, return_boxes=True, **kwargs)['preds']  # type:ignore[operator]
+            self.model(batch, return_preds=True, **kwargs)['preds']  # type:ignore[operator]
             for batch in processed_batches
         ]
-        return [pred for batch in predicted_batches for pred in zip(*batch)]
+        return [pred for batch in predicted_batches for pred in batch]
