@@ -5,11 +5,12 @@
 
 import logging
 import os
+from typing import Any, Callable, List, Optional, Union
 from zipfile import ZipFile
-from tensorflow.keras import layers, Model
-from typing import Callable, Optional, List, Any, Union
 
-from ..data_utils import download_from_url
+from tensorflow.keras import Model, layers
+
+from doctr.utils.data import download_from_url
 
 logging.getLogger("tensorflow").setLevel(logging.DEBUG)
 
